@@ -109,15 +109,110 @@
   return(ErrMsg)
 }
 
-.CheckLength <- function(z, s, d, y) {
+.CheckTau <- function(tau) {
+  ## for argument tau
+  ## 1. it must not be missing
+  ## 2. it must be numeric
+  ## 3. it must have at least 1 element
+  ## 4. it can have no more then 2 elements
+  ## 5. no elements may be NA
+  
+  if(missing(tau))
+    return("'tau' argument must be supplied")
+
+  ErrMsg <- NULL
+  if(!is.numeric(tau))
+    ErrMsg <- c(ErrMsg, "'tau' must be a numeric")
+
+  if(length(tau) < 1 || length(tau) > 2)
+    ErrMsg <- c(ErrMsg, "'tau' must be a vector of 1 or 2 elements")
+
+  if(any(is.na(tau)))
+    ErrMsg <- c(ErrMsg, "'tau' may not contain any NA values")
+
+  return(ErrMsg)
+}
+
+.CheckPhiPiPsi <- function(phi, Pi, psi, p0, p1) {
+  ## for arguments phi, Pi, and psi
+  ## 1. Only one of these argument may be specified
+  ## 2. the specified argument may not be missing
+  ## 3. the specified argument must have 1 or more values
+  ## 4. Pi must be in range of 0 to 1
+  ## 5. phi must be in range of 0 to 1
+  ## 6. phi, Pi, psi must be numeric
+
+  if(sum(c(missing(phi), missing(Pi), missing(psi))) == 0)
+    return("one of arguments 'phi', 'Pi', or 'psi' must be specified")
+
+  ErrMsg <- NULL
+  if(sum(c(missing(phi), missing(Pi), missing(psi)) > 1))
+    ErrMsg <- c(ErrMsg,
+                "only one of the arguments 'phi', 'Pi', or 'psi' may be specified")
+
+  if(!missing(phi) && !is.null(phi)) {
+    if(length(phi) < 1)
+      ErrMsg <- c(ErrMsg, "'phi' must have 1 or more elements")
+                  
+    if(any(is.na(phi)))
+      ErrMsg <- c(ErrMsg,
+                  "'phi' may not contain NA values")
+
+    if(!is.numeric(phi))
+      ErrMsg <- c(ErrMsg, "'phi' must be a numeric vector")
+    
+    if(!missing(p0) && !missing(p1) && any(phi > min(p0, p1)/p1 | phi < max(0, p0 + p1 - 1)/p1))
+        ErrMsg <- c(ErrMsg, "values of 'phi' must be constitant with data provided 'phi > min(p0, p1)/p1, phi < max(0, p0 + p1 - 1)/p1'")
+    else if(any(phi > 1 | phi < 0))
+        ErrMsg <- c(ErrMsg, "values of 'phi' must be between 0 and 1")
+
+  }
+    
+  if(!missing(Pi) && !is.null(Pi)) {
+    if(length(Pi) < 1)
+      ErrMsg <- c(ErrMsg, "'Pi' must have 1 or more elements")
+                  
+    if(any(is.na(Pi)))
+      ErrMsg <- c(ErrMsg,
+                  "'Pi' may not contain NA values")
+
+    if(!is.numeric(Pi))
+      ErrMsg <- c(ErrMsg, "'Pi' must be a numeric vector")
+
+    if(!missing(p0) && !missing(p1) && any(Pi > min(p0,p1) | Pi < max(0,p0 + p1 - 1)))
+      ErrMsg <- c(ErrMsg, "values of 'Pi' must be constitant with data provided 'Pi > min(p0,p1), Pi < max(0,p0 + p1 - 1)'")
+    else if(any(Pi > 1 | Pi < 0))
+      ErrMsg <- c(ErrMsg, "values of 'Pi' between 0 and 1")
+  }
+
+  if(!missing(psi) && !is.null(psi)) {
+    if(length(psi) < 1)
+      ErrMsg <- c(ErrMsg, "'psi' must have 1 or more elements")
+                  
+    if(any(is.na(psi)))
+      ErrMsg <- c(ErrMsg,
+                  "'psi' may not contain NA values")
+
+    if(!is.numeric(psi))
+      ErrMsg <- c(ErrMsg, "'psi' must be a numeric vector")
+  }
+
+  return(ErrMsg)
+}
+  
+.CheckLength <- function(z, s, d, y, v) {
   ## for vectors z, s, d, and y
   ## 1. must be same length
 
-  vectorMissing <- c(missing(z), missing(s), missing(d), missing(y))
+  vectorLengths <- c(if(!missing(z)) length(z),
+                     if(!missing(s)) length(s),
+                     if(!missing(d)) length(d),
+                     if(!missing(y)) length(y),
+                     if(!missing(v)) length(v))
   
-  if(!any(vectorMissing) &&
-     (length(z) != length(s) || length(z) != length(d) ||
-      length(z) != length(y)))
+  vectorMissing <- c(missing(z), missing(s), missing(d), missing(y), missing(v))
+  
+  if(any(!vectorMissing) && length(unique(vectorLengths)) > 1)
     return(sprintf("Vectors %s are of unequal lengths", paste(c('z','s','d','y')[!vectorMissing], collapse=',')))
 
   return(NULL)
@@ -134,7 +229,7 @@
     return("'z' argument must be supplied")
 
   ErrMsg <- NULL
-  if(any(is.na(z)) && na.rm)
+  if(any(is.na(z)) && !na.rm)
     ErrMsg <- c(ErrMsg,
                 "'z' cannot contain any NA values")
 
@@ -168,7 +263,7 @@
 
 
 
-.CheckY <- function(y, s, selection, ...) {
+.CheckY <- function(y, s, selection, na.rm, ...) {
   ## y must be
   ## 1. not missing
   ## 2. cannot be NA is s is selected.
@@ -177,8 +272,8 @@
     return("'y' argument must be supplied")
 
   ErrMsg <- NULL
-  if(!missing(selection) && !missing(s) && length(s) == length(y) &&
-     any(selection %in% s) &&
+  if(!missing(selection) && !missing(s) && !na.rm &&
+     length(s) == length(y) && any(selection %in% s) &&
      any(s %in% selection & !is.na(s) & is.na(y)))
     ErrMsg <- c(ErrMsg,
                 sprintf("argument 'y' cannont contain a NA value if the corresponding 's' is %s",
@@ -186,7 +281,7 @@
   return(ErrMsg)
 }
 
-.CheckD <- function(d, s, selection) {
+.CheckD <- function(d, s, selection, na.rm) {
   ## d must be
   ## 1. not missing
   ## 2. cannot be NA is s is selected.
@@ -195,8 +290,8 @@
     return("'d' argument must be supplied")
 
   ErrMsg <- NULL
-  if(!missing(selection) && !missing(s) && length(s) == length(d) &&
-     any(selection %in% s) &&
+  if(!missing(selection) && !missing(s) && !na.rm &&
+     length(s) == length(d) && any(selection %in% s) &&
      any(s %in% selection & !is.na(s) & is.na(d)))
     ErrMsg <- c(ErrMsg,
                 sprintf("argument 'd' cannot contain a NA value if the corresponding 's' is %s", paste(selection, collapse=",")))
@@ -204,80 +299,19 @@
   return(ErrMsg)
 }
 
-.ComputePiPsiPhi <- function(call = match.call(definition=sys.function(sys.parent()),
-                               call=sys.call(sys.parent())),
-                             envir=parent.frame(n=2),
-                             p0, p1) {
-  ## Assume only one of Pi, psi, or phi is present.  Calculate other two values.
-  .ComputeFunc <- function(psi, Pi, phi, ..., p0, p1){
-    if(!missing(psi) && !is.null(psi)) {
-      Pi <-
-        ifelse(abs(psi) < sqrt(.Machine$double.eps), p0*p1,
-               -(sqrt((p1^2-2*p0*p1+p0^2)*exp(2*psi)+p1^2
-                      +exp(psi)
-                      *(-2*p1^2+2*p1-2*p0^2+2*p0)
-                      +(2*p0-2)*p1+p0^2-2*p0+1)
-                 +p1+exp(psi)*(-p1-p0)+p0-1)
-               /(2*exp(psi)-2))
+.CheckV <- function(v, followup.time, na.rm) {
+  ## v must be
+  ## 1. present if followup.time is non-NULL or non-missing
+  ## 2. cannot be pressent if followup.time if NULL
+  ## 2. a numeric vector
+  ## 3. cannot contain NA unless na.rm == TRUE
+  if(missing(followup.time) || is.null(followup.time)) {
+    if(!missing(v))
+      return("'v' argument cannot be supplied without a 'followup.time' argument")
+  } else {
+    if(missing(v))
+      return("'followup.time' argument cannot be supplied without a 'v' argument")
 
-      phi <- Pi/p1
-    } else if(!missing(phi) && !is.null(phi)) {
-      Pi <- p1*phi
-      psi <- log((p1 * phi^2 + (1 - p0 - p1)*phi)/
-                 (p1 * phi^2 - (p1 + p0)* phi + p0))
-    } else {
-      psi <- log(Pi * (1 - p1 - p0 + Pi)/(p1 - Pi)/(p0 - Pi))
-      phi <- Pi/p1
-    }
-
-    return(list(psi = psi, Pi=Pi, phi=phi))
   }
-
-  call[[1]] <- .ComputeFunc
-  eval(call, envir=data.frame(p0=p0, p1=p1), enclos=envir)
-}
-
-.RunCheck <- function(checks,
-                      parentCall=match.call(definition=sys.function(sys.parent()),
-                        call=sys.call(sys.parent())),
-                      envir=parent.frame(n=2)) {
-
-  ErrMsg <- NULL
-  for(check in checks) {
-    if(is.character(check)) {
-      check <- as.symbol(check)
-    }
-
-    parentCall[[1]] <- check
-    ErrMsg <- c(ErrMsg, eval(parentCall, envir=envir))
-  }
-
-  return(ErrMsg)
-}
-
-.RunCompute <- function(check, ...,
-                        parentCall=match.call(definition=sys.function(sys.parent()),
-                          call=sys.call(sys.parent())),
-                        envir=parent.frame(n=2)) {
-    pNames <- names(parentCall)
-
-    if(is.null(pNames))
-      pNames <- ""
     
-    thisCall <- match.call(expand.dots=FALSE)
-
-    dotNames <- names(thisCall[['...']])
-    dotNames <- dotNames[dotNames != ""]
-    
-    ## Ensure no name overlap
-    if(any(dotNames  %in% names(parentCall))) {
-      parentCall[names(thisCall[['...']]) %in% names(parentCall)] <- NULL
-    }
-
-    parentCall[seq.int(from=length(parentCall)+1L,
-                       length.out=length(dotNames))] <- match.call()[dotNames]
-    names(parentCall) <- c(pNames, dotNames)
-
-    parentCall[[1]] <- as.name(check)
-    eval(parentCall, envir=parent.frame())
 }
